<?php

/* $Id: media_mover.inc,v 1.5.4.19 2007/07/01 14:58:38 arthuregg Exp $ */

/**
 * This file contains all the media mover functions for the media_mover hook
 * which are seperate from the internal workings of the media mover module
 *
 * all functions which are core to media mover which are not called from the
 * media mover hook should be in the media_mover.module file
 */


/**
 * Implementation of media_mover hook
 * @param $op is the operator to return
 * @param $action_id is which action is being called
 * @param $configuration is the specific configuration saved for the action for this configuration
 * @param $file is the file in use
 * @param $running_config is the full configuration data currently running 
 */
function media_mover_media_mover($op, $action = null, $configuration = null, &$file = array(), $running_config = null ) {

  switch ($op) {

    // give your module a distinct name
    case 'name':
      return "Media Mover module";
    break;

    // defines the actions that this module does
    // these are numbered for easy identification of what
    // action is being called
    case 'actions':
      return array( 
        1 => t('Bypass this operation'),
        2 => t('select drupal uploaded files'),
        3 => t('set node status'),
        4 => t('Save data as a node'),
        5 => t('Attach processd file to node'),
      );
    break;

    // create edit configuration option set
    // $action is used to ensure namespacing is kept consistent, form
    // elements must be named $form[{$action_id}--your_variable_name]
    // $configuration is an array, settings stored on a per config basis
    case 'config':
      switch ($action) {
        
        // setup the bypass options
        // these are only needed by this module- they create a bypass option
        // so that you can bypass portions of the media mover process
        case "harvest--1": //bypass
          return array("#title" => "Bypass");
        break;
        case "process--1": //bypass
          return array("#title" => "Bypass");
        break;
        case "storage--1": //bypass
          return array("#title" => "Bypass");
        break;
        case "complete--1": //bypass
          return array("#title" => "Bypass");
        break;
        
        // this calls the harvest
        case 'harvest--2': //select_drupal_uploaded_files
          return _media_mover_config_harvest($action, $configuration, $running_config);
        break;
        
        // this calls the completion 
        case 'complete--3': //set node status
          return _media_mover_config_complete($action, $configuration);
        break;
        
        // this calls the storage
        case 'storage--4':
          return _media_mover_config_save_node($action, $configuration);
        break;
        
        case 'storage--5':
          return _media_mover_config_attach_file($action, $configuration);
        break;                            
      }
      
    break;

    // set standard configuration for this module
    // this is the global config. Individual config items
    // should be in "new config"
    // displayed on admin/settings/media_mover
    // returns a form array
    case 'admin':
      return _media_mover_admin();
    break;

    // defines what type of driver this module is
    // array of harvest, process, storage, complete
    // a module may operate more than one verb
    case 'verbs':
      return array("harvest", "process", "storage", "complete");
    break;

    // defines directories (under master media_mover directory) this module uses
    // array of directories, will be created under the default
    // media_mover directory, set in admin
    case 'directories':
    break;

    // allows for module to add additional data in a different db per $file
    // called from media_mover's _mm_file_db_add
    // $action_id, $configuration, $file are available 
    case 'add':
    break;

    // allows for module to fetch additional data
    // added to $file to db action. called from media_mover's _mm_file_db_fetch and _mm_files_db_fetch
    // by default, medial_mover's files table is used. $file is available
    case 'fetch':
    break;

    // allows for module to update additional critera
    // add file to db action. called from media_mover's mm_files_db_update
    // by default, medial_mover's files table is used. $file is available
    case 'update':
    break;

    // functions called on harvest op
    // returns an array of $files
    case 'harvest':
      return _mm_harvest($action, $configuration, $running_config);
    break;

    // functions called on process op
    case 'process':
    break;

    // functions called on storage op
    case 'storage':    
      switch ($action) {      
        case '4':
          return _mm_node_save($file, $configuration);
        break;
        
        // save file to node attachments
        case '5':
          return media_mover_attach_file($file, $configuration);
        break;        
      }
    break;

    // functions called on completion
    case 'complete':
      return _mm_complete($configuration, $file);
    break;

    // custom theme a file assoicated with this module
    case 'theme':
    break;

    // implements a delete call from media mover
    case 'delete':
      switch ($action){ 
      
        // this calls the storage
        case 'storage--4':
          return;
        break;
        
        case '5':
          return media_mover_attach_file_delete($configuration);
        break;   
      }

    default:
      return;
    break;
  }
  return;
}


/* ************************************************ */
/* Media Mover: Media Mover hook  functions */
/* ************************************************ */

/**
 * Get list of files that match assumed critera
 *
 * returns a joined array from node and file tables
 */
function _mm_harvest($action, $configuration, $running_config) {
  
  // this builds a set of node types that is mysql friendly for n.type IN (  ...  )
  if ($node_types = $configuration['mm_node_types'] ) {
    foreach ($node_types as $type) {
      if ($type) {
        $node_type_list[] = "'". $type ."'";
      }
    }
    // make sure $node_type_list is populated
    if ($node_type_list) {
      $node_type_list = implode(" ," , $node_type_list) ;
    }
    else {
      // exit if there are no node types selected
      return; 
    }

    // get all potentially harvestable files
    // select all files join with nodes of $node_type_list 
    // where node changed date is greater than last run start time
    
    $results = db_query("SELECT * FROM {files} f " .
      "LEFT JOIN {node} n ON f.nid = n.nid ".
      "WHERE ((n.type IN ($node_type_list)) AND (n.changed > %d)) ORDER BY n.changed DESC", $configuration['last_start_time']);

    // take each result and check to see if this FID is stored already
    while ($result = db_fetch_array($results)) {

      $dupe = db_result(db_query('SELECT mmfid FROM {media_mover_files} WHERE fid= %d AND cid = %d', $result['fid'], $running_config->cid));
    
      // this file has not been harvested by this configuration, continue
      if (! $dupe) {

        // @ TODO check what kind of file it is (configuration?)
        if (_mm_harvest_file_type($result, $configuration) ){          
          $afile = $_SERVER['DOCUMENT_ROOT'] . substr(base_path(), 1) . $result['filepath'];
          if (file_exists($afile)){  
            // now we harvest file
            $result['filepath'] = $afile;
            $files['harvest_module'] = $running_config->harvest->module;
            $files['harvest_action'] = $running_config->harvest->action;
            $files[] = $result;
          }          
        }        
      }    
    }
  }
  return $files;
}

/**
 * checks incoming file against the options set in the configuration
 * this only checks the file extension
 */
function _mm_harvest_file_type($file, $configuration){
  $types = str_replace(" ", "", $configuration['file_types']);
  $types = explode(',', $types);

  // get the file extension
  $path_parts = pathinfo($file['filepath']);
  $extension = $path_parts['extension']; 
  
  // check if file extension is supported
  foreach ($types as $type) {
    if (stristr($type, $extension)) { return true; }     
  }
  return false;  
}



/**
 * Get list of files that match assumed critera
 *
 * returns a joined array from node and file tables
 */
function _mm_complete( $configuration, $file) {
  if ($configuration['complete_conditions']['published'] != '0') { $options[] = "status = 1 ";}
  if ($configuration['complete_conditions']['in moderation queue'] != '0') { $options[] = "moderate = 1 ";}
  if ($configuration['complete_conditions']['promoted to front page'] != '0') { $options[] = "promote = 1 ";}
  $options = implode(", ", $options);
  db_query("UPDATE node SET $options WHERE nid = %d", $file['nid']);
}


/**
 * stores the actual data as a node
 * @ TODO finish up this functionality to deploy
 *        requires creating the data store
 */
function _mm_node_save($file, $configuration) {
  // get node type from configuration to store

  $account = theme('media_mover_node_save_user_map', $file, $file['data']['user'], $configuration);
  
  $node = theme('media_mover_node_save_map', $file, $file['data']['node'], $configuration, $account);
    
  // save node
  node_save($node);

  // save file data to files table
  
  // get the next $fid
  $fid = db_next_id('{files}_fid');
  
  // get drupal file path  
  if ($file['data']['file']['drupal_path']) {
    $drupal_path = $file['data']['file']['drupal_path'];
  }
  else {
    $drupal_path = substr($file['process_file'], (strlen(base_path()) + strpos($file['process_file'], base_path())));        
  }
  
  // insert file
  db_query("INSERT INTO {files} (fid, nid, filename, filepath, filemime, filesize) VALUES (%d, %d, '%s', '%s', '%s', %d)", $fid, $node->nid, basename($file['process_file']), $drupal_path, $file['mime'], filesize($file['process_file']));
  db_query("INSERT INTO {file_revisions} (fid, vid, list, description) VALUES (%d, %d, %d, '%s')", $fid, $node->nid, 0, basename($file['process_file']));
    
  // add the storage data back to the file
  $file['storage_file'] = $file['data']['file']['drupal_path'];
  $file['storage_module'] = 'media_mover';
  $file['storage_action'] = 'attach file to node';
  
  return $file;
}


/**
 * attach an incoming file to a node
 * @param $file is a file array
 * @param $configuration is this functions configuration
 * @return $file array
 */
function media_mover_attach_file($file, $configuration) {
  
  // get drupal file path  
  if ($file['data']['file']['drupal_path']) {
    $drupal_path = $file['data']['file']['drupal_path'];
  }
  else {
    $drupal_path = substr($file['process_file'], (strlen(base_path()) + strpos($file['process_file'], base_path())));        
  }
  
  // if $drupal_path isn't set, we have no file data to save at this point... 
  if (! $drupal_path ) { return; }
  
  
  // get mime type
  if (! $mime = $file['data']['file']['mime']) {
     $path_parts = pathinfo($drupal_path);
     $mime = $path_parts['extension'];   
  }
  
  // get the next $fid
  $fid = db_next_id('{files}_fid');
  
  // insert file
  db_query("INSERT INTO {files} (fid, nid, filename, filepath, filemime, filesize) VALUES (%d, %d, '%s', '%s', '%s', %d)", $fid, $file['nid'], basename($file['process_file']), $drupal_path, $file['mime'], filesize($file['process_file']));
  db_query("INSERT INTO {file_revisions} (fid, vid, list, description) VALUES (%d, %d, %d, '%s')", $fid, $file['nid'], 0, basename($file['process_file']));
    
  // add the storage data back to the file
  $file['storage_file'] = $file['data']['file']['drupal_path'];
  $file['storage_module'] = 'media_mover';
  $file['storage_action'] = 'attach file to node';
  
  return $file;
}


/**
 * removes references to files 
 * @ TODO
 */
function media_mover_attach_file_delete($configuration) {
}

/* ************************************************** */
/* Media Mover: Media Mover config & admin  functions */
/* ************************************************** */

/**
 * Admin form for media mover
 * @ TODO move the types configuration over to the individual configs
 */
function _media_mover_admin() {

  $form['mm_config']['mm_file_dir'] = array(
   '#type' => 'textfield',
   '#title' => t('Converted files storage'),
   '#description' => t('Sets the directory where media_mover files are stored.'),
   '#default_value' =>  variable_get('mm_file_dir', file_create_path("media_mover")),
  );

  return $form;
}

/**
 * configuration options for attaching a file to a node
 */
function _media_mover_config_attach_file($action_id, $configuration){
  $form['mm_attach_file'] = array(  
    '#title' => t('Attach file to origional node'),
    '#type' => 'fieldset',
    '#value' => '',
    '#description' => t('There are no configuration options for this'),
    '#collapsed' => false,
    // id => modulename--action this allows the javascript auto hide and show on the add config page
    // class => mm_config_option allows jQuery to close and open this easily
    '#attributes' => array('id' => 'media_mover--5', 'class' => 'mm_config_option'),
  );
  return $form;
}

/**
 * Config config form for media mover
 * creates the options for each configuration instance
 *
 * NOTE: name form files like: ["{$action}--db_options"]
 * so that they are processed correctly on the add configuration
 * screen
 */
function _media_mover_config_harvest($action_id, $configuration) {

  $form["mm_harvest_conf"] = array(
    '#type' => 'fieldset',
    '#title' => t('Harvest Configuration'),
    '#description' => t('This module harvests files attached to nodes that have been created on the site.'),
    '#collapsed' => false,
    // id => modulename--action this allows the javascript auto hide and show on the add config page
    // class => mm_config_option allows jQuery to close and open this easily
    '#attributes' => array('id' => 'media_mover--2', 'class' => 'mm_config_option'),
  );

  foreach (node_get_types() as $type) {
    $node_types[$type->type] = $type->name;
  }

  $form['mm_harvest_conf']["{$action_id}--mm_node_types"] = array(
   '#type' => 'checkboxes',
   '#title' => t('Select files from these content types'),
   '#description' => t('Select content types you wish to search for video files to convert.'),
   '#options' => $node_types,
   '#default_value' => $configuration['mm_node_types'],
  );

  $form['mm_harvest_conf']["{$action_id}--file_types"] = array(
    '#title' => t('File types'),
    '#type' => "textfield",
    '#default_value' => $configuration['file_types'] ? $configuration['file_types'] : 'avi,mov,wmv,mpeg,mp4,mpeg2,dv',
    '#description' => t('Comma seperated list of file types that can be harvested. These are file extensions and are values like "avi,mov,wmv,mpeg,mp4,mpeg2,dv".'),
    );
  return $form;
}


/**
 * Config config form for media mover
 * creates the options for each configuration instance
 *
 * NOTE: name form files like: ["{$action}--db_options"]
 * so that they are processed correctly on the add configuration
 * screen
 */
function _media_mover_config_complete($action, $configuration) {
  $form["complete"] = array(
    '#type' => 'fieldset',
    '#title' => t('Completion Configuration'),
    '#collapsed' => false,
    // id => modulename--action this allows the javascript auto hide and show on the add config page
    // class => mm_config_option allows jQuery to close and open this easily
    '#attributes' => array ('id' => 'media_mover--3', 'class' => 'mm_config_option'),
  );

  $options = array('published' => t("Published"), "in moderation queue" => t("In moderation queue"), "promoted to front page" => t("Promoted to front page"));
  $form['complete']["{$action}--complete_conditions"] = array(
    '#title' => t('Completion condition'),
    '#type' => "checkboxes",
    '#options' => $options,
    '#default_value' => $configuration['complete_conditions'],
    '#description' => t('Set the status of the node post completion'),
    );
  return $form;
}

/**
 * config form for the storage option
 */
function _media_mover_config_save_node($action, $configuration) {
  global $user;
  // get node types
  $types = node_get_types();
  
  foreach ($types as $type) {
    $options[$type->type] = $type->name;
  }
  
  $form["storage"] = array(
    '#type' => 'fieldset',
    '#title' => t('Node storage configuration'),
    '#collapsed' => false,
    // id => modulename--action this allows the javascript auto hide and show on the add config page
    // class => mm_config_option allows jQuery to close and open this easily
    '#attributes' => array ('id' => 'media_mover--4', 'class' => 'mm_config_option'),
    '#description' => t('This module will try to do a best guess of harvested data and save this to a node.'),
  );

  $form['storage']["{$action}--node_save_type"] = array(
    '#title' => t('Default node type'),
    '#type' => "select",
    '#options' => $options,
    '#default_value' => $configuration['node_save_type'],
    '#description' => t('Save the harvested and processed data on this node type'),
  );
  
  $form['storage']["{$action}:node_save_override"] = array(
    '#title' => t('Data override'),
    '#type' => "checkbox",
    '#default_value' => $configuration['node_save_override'] ,
    '#description' => t('If this is checked, all saved nodes will have the title an author set below.'),
  );
  
  $form['storage']["{$action}--node_save_name"] = array(
    '#type' => 'textfield',
    '#title' => t('Default author'),
    '#maxlength' => 60,
    '#autocomplete_path' => 'user/autocomplete',
    '#default_value' => $configuration['node_save_name'] ? $configuration['node_save_name'] : $user->name,
    '#description' => t('If a user is not passed in, this will be the default author.'),
  );
 
  $form['storage']["{$action}--node_title_default"] = array(
    '#type' => 'textfield',
    '#title' => t('Default title'),
    '#maxlength' => 60,
    '#default_value' => $configuration['node_title_default'],
    '#description' => t('If a title is not passed in, this will be the default node title.'),
  );
  return $form;
}

/* ***************************************** */
/* theme functions */
/* ***************************************** */

/**
 * this function lets users map incoming data to node values
 * for advanced node data saving. User data is handled by
 * _mm_node_save
 * 
 * @param $file is the complete file array
 * @param $node_data is the array of data stored in $file['data']['node']
 * @param $configuration contains the default settings
 * @param $account is a drupal user object
 * @return a node object for saving
 */
function theme_media_mover_node_save_map( $file, $node_data, $configuration, $account) {

    // @ TODO check taxonomy
  
  // set node type
  $node->type = $configuration['node_save_type'];

  
  if ($configuration['node_save_override'] ) {
    $node->title = $configuration['node_save_title'];    
  }
  else { 
    // set the title
    $node->title = $node_data['title'] ? $node_data['title'] : $configuration['node_title_default'];
    unset($node_data['title']);
  }
  
  // set the body
  $node->body = $node_data['body'];
  unset($node_data['body']);
  
  
  // @ TODO make this much more friendly to cck
  if ($node_data) {
    foreach ($node_data as $field_name => $field_value) {
      if ($field_name != 'title' || $field_name != 'body') {
        $node->body .= '<h1>'. $field_name .'</h1>'. $field_value;      
      }    
    }
  }
  
  // attach the user data
  $node->uid = $account->uid;
  
  return $node;
}

/**
 * this function lets users map incoming data to node values
 * for advanced node data saving. User data is handled by
 * _mm_node_save
 * 
 * @data is the array of data stored in $file['data']['node']
 * @node is the node object with some user data
 * @configuration contains the default settings
 * @return a node object for saving
 */
function theme_media_mover_node_save_user_map($file, $user, $configuration) {
  
  // check the override  
  if ($configuration['node_save_override'] ) {
    $account = user_load(array('name' => $configuration['node_save_name']));
  }
  // check if there is user/author data associated with this file
  elseif ($file['data']['user']) { 
    
    $passed_user = $file['data']['user'];
    
    // check if there is a drupal uid
    if ($passed_user['uid']) {
      $account = user_load(array('uid' => $passed_user['uid']));
    }
  
    // is an email address being passed in?
    elseif ($passed_user['email']) {
      $account = user_load(array('mail' => $passed_user['email']));    
    }
    
    // is a user name being passed in?
    elseif ($file['data']['user']['name']) {
      $account = user_load(array('name' => $passed_user['name']));    
    }             
  }

  // make sure we have a user
  if (! $account) {
    $account = user_load(array('name' => $configuration['node_save_name']));    
  }
  
  return $account;
  
}
